"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var ReadsService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReadsService = void 0;
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const influxdb_client_1 = require("@influxdata/influxdb-client");
const unit_enum_1 = require("./unit.enum");
const date_utils_1 = require("../date.utils");
let ReadsService = ReadsService_1 = class ReadsService {
    constructor(configService) {
        this.configService = configService;
        this.logger = new common_1.Logger(ReadsService_1.name);
    }
    async onModuleInit() {
        var _a;
        const url = this.configService.get('INFLUXDB_URL');
        const token = this.configService.get('INFLUXDB_TOKEN');
        const organization = (_a = this.configService.get('INFLUXDB_ORG')) !== null && _a !== void 0 ? _a : '';
        const bucket = this.configService.get('INFLUXDB_BUCKET');
        if (!url) {
            throw new Error('Missing INFLUXDB_URL parameter');
        }
        if (!token) {
            throw new Error('Missing INFLUXDB_TOKEN parameter');
        }
        if (!bucket) {
            throw new Error('Missing INFLUXDB_BUCKET parameter');
        }
        this.logger.debug(`Using InfluxDB instance on ${url}`);
        this.bucket = bucket;
        this.organization = organization;
        this.config = {
            url,
            token,
        };
    }
    async store(meterId, measurement) {
        const multiplier = this.getMultiplier(measurement.unit);
        const points = measurement.reads.map((m) => new influxdb_client_1.Point('read')
            .tag('meter', meterId)
            .intField('read', m.value * multiplier)
            .timestamp(new Date(m.timestamp)));
        const writer = this.dbWriter;
        writer.writePoints(points);
        await writer.close();
    }
    async aggregate(meterId, filter) {
        try {
            const query = `
      from(bucket: "${this.bucket}")
      ${this.getRangeFilter(filter.start, filter.end)}
      |> filter(fn: (r) => r.meter == "${meterId}" and r._field == "read")
      ${filter.difference ? '|> difference()' : ''}
      |> window(every: ${filter.window})
      |> ${filter.aggregate}()
      `;
            return this.executeAggregated(query);
        }
        catch (e) {
            this.logger.error(e.message);
            throw e;
        }
    }
    async find(meterId, filter) {
        try {
            const query = this.findByMeterQuery(meterId, filter);
            return this.execute(query);
        }
        catch (e) {
            this.logger.error(e.message);
            throw e;
        }
    }
    async findLatestRead(meterId, startDuration) {
        try {
            const query = this.findLatestReadByMeterQuery(meterId, startDuration);
            const reads = await this.execute(query);
            if (reads.length === 0) {
                throw new common_1.NotFoundException(`Unable to get the latest reading. There are no readings yet for meter ${meterId}`);
            }
            return reads[0];
        }
        catch (e) {
            this.logger.error(e.message);
            throw e;
        }
    }
    async findDifference(meterId, filter) {
        try {
            const query = `${this.findByMeterQuery(meterId, filter)}
      |> difference()
      `;
            return this.execute(query);
        }
        catch (e) {
            this.logger.error(e.message);
            throw e;
        }
    }
    findByMeterQuery(meterId, filter) {
        return `
    from(bucket: "${this.bucket}")
    ${this.getRangeFilter(filter.start, filter.end)}
    |> limit(n: ${filter.limit}, offset: ${filter.offset})
    |> filter(fn: (r) => r.meter == "${meterId}" and r._field == "read")
    `;
    }
    getRangeFilter(start, end) {
        const endPlusOneMs = (0, date_utils_1.addMilliseconds)(new Date(end), 1);
        return `
      |> range(start: ${start}, stop: ${endPlusOneMs.toISOString()})
      |> filter(fn: (r) => r._time >= ${start} and r._time <= ${end})
    `;
    }
    findLatestReadByMeterQuery(meterId, startDuration) {
        const start = startDuration ? `-${startDuration}` : '-1d';
        return `
    from(bucket: "${this.bucket}")
    |> range(start: ${start}, stop: now())
    |> filter(fn: (r) => r.meter == "${meterId}" and r._field == "read")
    |> last()
    `;
    }
    async execute(query) {
        const data = await this.dbReader.collectRows(query);
        return data.map((record) => ({
            timestamp: new Date(record._time),
            value: Number(record._value),
        }));
    }
    async executeAggregated(query) {
        const data = await this.dbReader.collectRows(query);
        return data.map((record) => ({
            start: new Date(record._start),
            stop: new Date(record._stop),
            value: Number(record._value),
        }));
    }
    get dbWriter() {
        return new influxdb_client_1.InfluxDB(this.config).getWriteApi(this.organization, this.bucket);
    }
    get dbReader() {
        return new influxdb_client_1.InfluxDB(this.config).getQueryApi(this.organization);
    }
    getMultiplier(unit) {
        switch (unit) {
            case unit_enum_1.Unit.Wh:
                return 1;
            case unit_enum_1.Unit.kWh:
                return 10 ** 3;
            case unit_enum_1.Unit.MWh:
                return 10 ** 6;
            case unit_enum_1.Unit.GWh:
                return 10 ** 9;
        }
    }
};
ReadsService = ReadsService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [config_1.ConfigService])
], ReadsService);
exports.ReadsService = ReadsService;
//# sourceMappingURL=reads.service.js.map